@@ -6,6 +6,65 @@ Still to define if it will be kept
 
 @author: Mike - Manfredda
 """
+from enum import Enum as StandardEnum
+
+#==============================================================================
+#  Class Enum
+#==============================================================================
+class Enum(StandardEnum):
+	'''
+	Decorator (used as subclass) for making the comparisong between enums behave
+	as we want.
+	'''
+	
+	def __eq__(self, other):
+		'''
+		Useless here: it should be applied to any member
+		'''
+		if self.value == other.value:
+			return True
+		else:
+			return False
+		
+#	@classmethod
+#	def ListRoles(cls):
+#		role_names = [member.value for role, member in cls.__members__.items()]
+#		return role_names
+#
+#		
+		
+#==============================================================================
+#  Class LogBuffer
+#==============================================================================
+class LogBuffer():
+	'''
+	Used to make any class capable of carrying debug information.
+	Does not require intitialization.
+	Example:
+	a = LogBuffer()
+	a._LogBuffer__Add('s')
+	a._LogBuffer__Add('ss')
+	a._LogBuffer__LogBuffer
+	a._LogBuffer__Str
+	a._LogBuffer__Print()
+	'''
+	def __init__(self):
+		self.__LogBuffer = []
+
+	def __Add(self,x : str):
+		try:
+			self.__LogBuffer.append(x)
+		except:
+			self.__init__()
+			self._LogBuffer.append(x)
+			
+	
+	@property
+	def __Str(self):
+		return '\n'.join(self.__LogBuffer)
+
+	def __Print(self):
+		print(self.__Str)	
 
 #==============================================================================
 #  Exists
@@ -21,6 +80,100 @@ def Exists(var):
 		return False
 	   # Do something.
 
+def SetAttr(x, StrAttr : str, Value):
+	
+	TokList = StrAttr.split('.')
+	if len(TokList) ==1:
+		setattr(x, TokList[0], Value)
+	
+	elif len(TokList)>1:
+		
+		for Tok in TokList:
+			y = getattr(x,Tok)
+			GeldedTokList = '.'.join(TokList[1:])
+			print(GeldedTokList)
+			SetAttr(y,GeldedTokList, Value)
+
+#==============================================================================
+#  FUN: GetTheNotNone
+#==============================================================================
+def GetTheNotNone(ArgList):
+	'''
+	Return the first element which is not None.
+	Used in certain case for parameter handling.
+	It can be used ia try...except structure
+	
+	try:
+		c = GetTheNotNone([a,b])
+	except:
+		c = a
+		
+	'''
+	
+	for _ in ArgList:
+		if _ is not None:
+			return _
+	return None
+
+#==============================================================================
+#  FUN: IsArrayLike
+#==============================================================================
+def IsArrayLike(x):
+	import numpy as np
+	if type(x) == list:
+		return True
+	elif type(x) == np.ndarray:
+		return True
+	else:
+		return False
+
+#==============================================================================
+#  FUN: IsValidArrayLike
+#==============================================================================
+def IsValidArray(x, Complex = False):
+	'''
+	Check that all the values of x are float, with no nan, no string, no inf
+	
+	'''
+	import numpy as np
+	if Complex:
+		return np.any(np.iscomplex(x))
+	else:
+		return np.any(np.isreal((x)))
+
+
+#==============================================================================
+#  FUN: IsValidPythonName
+#==============================================================================	
+def IsValidPythonName(x):
+	try:
+		exec('%s = 42' % x)
+		return True
+	except:
+		return False	
+#==============================================================================
+#  FUN: MergeDictionaries
+#==============================================================================	
+def IsValidPythonName(x):
+	try:
+		exec('%s = 42' % x)
+		return True
+	except:
+		return False
+
+#==============================================================================
+#  Class: FrozenClass
+#==============================================================================
+class FrozenClass(object):
+    __isfrozen = False
+    def __setattr__(self, key, value):
+        if self.__isfrozen and not hasattr(self, key):
+            raise TypeError( "%r is a frozen class" % self )
+        object.__setattr__(self, key, value)
+
+    def _freeze(self):
+        self.__isfrozen = True
+		
 #==============================================================================
 #  Class: DataContainer
 #==============================================================================
@@ -363,4 +516,7 @@ def ListClassPropsStr(myClass, strOwnerClass = ''):
 def ListClassObjects(myClass, strOwnerClass = ''):
     strAttr = [iAttr for iAttr in dir(myClass) if not callable(iAttr) and not iAttr.startswith("__")]
     objAttr = [getattr(myClass,iAttr) for iAttr in strAttr]
-    return objAttr
\ No newline at end of file
+    return objAttr
+
+
+#SetAttr(a, 'b.c.d',  11)
\ No newline at end of file
